@@ -1,6 +1,6 @@
 /*
 	acr38cmd.c: Emulated CCID reader commands for ACR38 non-CCID reader
-	Copyright (C) 2011-2014   Advanced Card Systems Ltd.
+	Copyright (C) 2011-2015   Advanced Card Systems Ltd.
 
 	This library is free software; you can redistribute it and/or
 	modify it under the terms of the GNU Lesser General Public
@@ -112,7 +112,7 @@ RESPONSECODE ACR38_CmdPowerOn(unsigned int reader_index, unsigned int *nlength,
 	unsigned int length;
 	RESPONSECODE return_value = IFD_SUCCESS;
 	_ccid_descriptor *ccid_descriptor = get_ccid_descriptor(reader_index);
-	unsigned char option;
+	unsigned char option = ACR38_OPTION_NONE;
 
 	/* store length of buffer[] */
 	length = *nlength;
@@ -436,7 +436,7 @@ RESPONSECODE ACR38_Receive(unsigned int reader_index,
 
 	if (cmd[ACR38_STATUS_OFFSET] != 0)
 	{
-		ccid_error(cmd[ACR38_STATUS_OFFSET], __FILE__, __LINE__, __FUNCTION__);
+		acr38_error(cmd[ACR38_STATUS_OFFSET], __FILE__, __LINE__, __FUNCTION__);
 		return IFD_COMMUNICATION_ERROR;
 	}
 
