@@ -1,7 +1,7 @@
 /*
     ccid.c: CCID common code
     Copyright (C) 2003-2010   Ludovic Rousseau
-    Copyright (C) 2009-2014   Advanced Card Systems Ltd.
+    Copyright (C) 2009-2015   Advanced Card Systems Ltd.
 
     This library is free software; you can redistribute it and/or
     modify it under the terms of the GNU Lesser General Public
@@ -19,7 +19,7 @@
 */
 
 /*
- * $Id: ccid.c 6976 2014-09-04 11:35:46Z rousseau $
+ * $Id$
  */
 
 #include <config.h>
@@ -78,19 +78,7 @@ int ccid_open_hack_pre(unsigned int reader_index)
 			ccid_descriptor->readTimeout = 60*1000; /* 60 seconds */
 			break;
 
-		case GEMPCTWIN:
-		case GEMPCKEY:
-		case DELLSCRK:
-			/* Only the chipset with firmware version 2.00 is "bogus"
-			 * The reader may send packets of 0 bytes when the reader is
-			 * connected to a USB 3 port */
-			if (0x0200 == ccid_descriptor->IFD_bcdDevice)
-			{
-				ccid_descriptor->zlp = TRUE;
-				DEBUG_INFO1("ZLP fixup");
-			}
-			break;
-
+		case OZ776:
 		case OZ776_7772:
 			ccid_descriptor->dwMaxDataRate = 9600;
 			break;
@@ -235,8 +223,8 @@ static void set_gemalto_firmware_features(unsigned int reader_index)
 		unsigned int len_features = sizeof *gf_features;
 		RESPONSECODE ret;
 
-		ret = CmdEscape(reader_index, cmd, sizeof cmd,
-			(unsigned char*)gf_features, &len_features, 0);
+		ret = CmdEscapeCheck(reader_index, cmd, sizeof cmd,
+			(unsigned char*)gf_features, &len_features, 0, TRUE);
 		if ((IFD_SUCCESS == ret) &&
 		    (len_features == sizeof *gf_features))
 		{
@@ -577,11 +565,15 @@ int ccid_open_hack_post(unsigned int reader_index)
 
 		case ACS_APG8201:
 		case ACS_APG8201Z:
+		case ACS_APG8201Z2:
 			ccid_descriptor->wLcdLayout = 0x0210;
 
 			// APG8201 uses short APDU exchange
-			ccid_descriptor->dwFeatures &= ~CCID_CLASS_EXCHANGE_MASK;
-			ccid_descriptor->dwFeatures |= CCID_CLASS_SHORT_APDU;
+			if (ccid_descriptor->dwFeatures & CCID_CLASS_TPDU)
+			{
+				ccid_descriptor->dwFeatures &= ~CCID_CLASS_EXCHANGE_MASK;
+				ccid_descriptor->dwFeatures |= CCID_CLASS_SHORT_APDU;
+			}
 			break;
 
 		case ACS_ACR85_PINPAD_READER_ICC:
@@ -824,7 +816,8 @@ int ccid_open_hack_post(unsigned int reader_index)
  *					ccid_error
  *
  ****************************************************************************/
-void ccid_error(int error, const char *file, int line, const char *function)
+void ccid_error(int log_level, int error, const char *file, int line,
+	const char *function)
 {
 #ifndef NO_LOG
 	const char *text;
@@ -943,7 +936,7 @@ void ccid_error(int error, const char *file, int line, const char *function)
 			text = var_text;
 			break;
 	}
-	log_msg(PCSC_LOG_ERROR, "%s:%d:%s %s", file, line, function, text);
+	log_msg(log_level, "%s:%d:%s %s", file, line, function, text);
 #endif
 
 } /* ccid_error */
