@@ -1,7 +1,7 @@
 /*
     ccid.h: CCID structures
     Copyright (C) 2003-2010   Ludovic Rousseau
-    Copyright (C) 2009-2014   Advanced Card Systems Ltd.
+    Copyright (C) 2009-2015   Advanced Card Systems Ltd.
 
     This library is free software; you can redistribute it and/or
     modify it under the terms of the GNU Lesser General Public
@@ -19,7 +19,7 @@
 */
 
 /*
- * $Id: ccid.h 6922 2014-06-16 13:55:33Z rousseau $
+ * $Id$
  */
 
 #ifdef __APPLE__
@@ -147,11 +147,6 @@ typedef struct
 	 */
 	struct GEMALTO_FIRMWARE_FEATURES *gemalto_firmware_features;
 
-	/*
-	 * Zero Length Packet fixup (boolean)
-	 */
-	char zlp;
-
 	// Pointer to array of bStatus
 	unsigned char *bStatus;
 
@@ -250,6 +245,7 @@ typedef struct
 #define HP_CCIDSMARTCARDKEYBOARD 0x03F00036
 #define KOBIL_IDTOKEN 0x0D46301D
 #define FUJITSUSMARTKEYB 0x0BF81017
+#define FEITIANR502DUAL 0x096E060D
 
 // CCID readers
 #define ACS_ACR32_ICC_READER			0x072fb301
@@ -294,6 +290,7 @@ typedef struct
 #define ACS_AET62_1SAM_PICC_READER		0x072f0103
 #define ACS_APG8201						0x072f8201
 #define ACS_APG8201Z					0x072f8202
+#define ACS_APG8201Z2					0x072f8205
 
 // non-CCID readers
 #define ACS_ACR38U						0x072f9000
@@ -335,7 +332,8 @@ typedef struct
 
 int ccid_open_hack_pre(unsigned int reader_index);
 int ccid_open_hack_post(unsigned int reader_index);
-void ccid_error(int error, const char *file, int line, const char *function);
+void ccid_error(int log_level, int error, const char *file, int line,
+	const char *function);
 _ccid_descriptor *get_ccid_descriptor(unsigned int reader_index);
 void acr38_error(int error, const char *file, int line, const char *function);
 void EnablePicc(unsigned int reader_index, int enabled);
