@@ -1,7 +1,7 @@
 /*
 	ccid_usb.c: USB access routines using the libusb library
 	Copyright (C) 2003-2010	Ludovic Rousseau
-	Copyright (C) 2009-2014	Advanced Card Systems Ltd.
+	Copyright (C) 2009-2015	Advanced Card Systems Ltd.
 
 	This library is free software; you can redistribute it and/or
 	modify it under the terms of the GNU Lesser General Public
@@ -19,14 +19,13 @@
 */
 
 /*
- * $Id: ccid_usb.c 6975 2014-09-04 11:33:05Z rousseau $
+ * $Id$
  */
 
 #define __CCID_USB__
 
 #include <stdio.h>
 #include <string.h>
-#include <errno.h>
 #include <unistd.h>
 # ifdef S_SPLINT_S
 # include <sys/types.h>
@@ -129,6 +128,9 @@ typedef struct
 
 	// Max packet size of bulk out endpoint
 	int bulkOutMaxPacketSize;
+
+	// For a workaround for ACR122U
+	int last_write_size;
 } _usbDevice;
 
 /* The _usbDevice structure must be defined before including ccid_usb.h */
@@ -245,6 +247,7 @@ status_t OpenUSBByName(unsigned int reader_index, /*@null@*/ char *device)
 #else
 	/* 100 ms delay */
 	struct timespec sleep_time = { 0, 100 * 1000 * 1000 };
+	int count_libusb = 10;
 #endif
 	int interface_number = -1;
 	int i;
@@ -331,15 +334,14 @@ status_t OpenUSBByName(unsigned int reader_index, /*@null@*/ char *device)
 		rv = libusb_init(&ctx);
 		if (rv != 0)
 		{
-			DEBUG_CRITICAL2("libusb_init failed: %d", rv);
+			DEBUG_CRITICAL2("libusb_init failed: %s", libusb_error_name(rv));
 			return_value = STATUS_UNSUCCESSFUL;
 			goto end1;
 		}
 	}
 
 #ifdef __APPLE__
-	/* give some time to libusb to detect the new USB devices on Mac OS X */
-	nanosleep(&sleep_time, NULL);
+again_libusb:
 #endif
 	cnt = libusb_get_device_list(ctx, &devs);
 	if (cnt < 0)
@@ -424,7 +426,8 @@ status_t OpenUSBByName(unsigned int reader_index, /*@null@*/ char *device)
 
 				/* simulate a composite device as when libudev is used */
 				if ((GEMALTOPROXDU == readerID)
-					|| (GEMALTOPROXSU == readerID))
+					|| (GEMALTOPROXSU == readerID)
+					|| (FEITIANR502DUAL == readerID))
 				{
 						/*
 						 * We can't talk to the two CCID interfaces
@@ -576,8 +579,8 @@ status_t OpenUSBByName(unsigned int reader_index, /*@null@*/ char *device)
 				r = libusb_open(dev, &dev_handle);
 				if (r < 0)
 				{
-					DEBUG_CRITICAL4("Can't libusb_open(%d/%d): %d",
-						bus_number, device_address, r);
+					DEBUG_CRITICAL4("Can't libusb_open(%d/%d): %s",
+						bus_number, device_address, libusb_error_name(r));
 
 					continue;
 				}
@@ -602,8 +605,9 @@ again:
 						if (r < 0)
 						{
 							(void)libusb_close(dev_handle);
-							DEBUG_CRITICAL4("Can't set configuration on %d/%d: %d",
-									bus_number, device_address, r);
+							DEBUG_CRITICAL4("Can't set configuration on %d/%d: %s",
+									bus_number, device_address,
+									libusb_error_name(r));
 							continue;
 						}
 					}
@@ -614,8 +618,8 @@ again:
 					{
 #endif
 						(void)libusb_close(dev_handle);
-						DEBUG_CRITICAL4("Can't get config descriptor on %d/%d: %d",
-							bus_number, device_address, r);
+						DEBUG_CRITICAL4("Can't get config descriptor on %d/%d: %s",
+							bus_number, device_address, libusb_error_name(r));
 						continue;
 					}
 #ifdef __APPLE__
@@ -662,7 +666,7 @@ again:
 					/* an interface was specified and it is not the
 					 * current one */
 					DEBUG_INFO3("Found interface %d but expecting %d",
-						interface_number, interface);
+						interface, interface_number);
 					DEBUG_INFO3("Wrong interface for USB device %d/%d."
 						" Checking next one.", bus_number, device_address);
 
@@ -676,8 +680,8 @@ again:
 				if (r < 0)
 				{
 					(void)libusb_close(dev_handle);
-					DEBUG_CRITICAL4("Can't claim interface %d/%d: %d",
-						bus_number, device_address, r);
+					DEBUG_CRITICAL4("Can't claim interface %d/%d: %s",
+						bus_number, device_address, libusb_error_name(r));
 					claim_failed = TRUE;
 					interface_number = -1;
 					continue;
@@ -702,7 +706,7 @@ again:
 
 				/* reset for a next reader */
 				if (static_interface > 2)
-					static_interface = 1;
+					static_interface = (FEITIANR502DUAL == readerID) ? 0: 1;
 #endif
 
 				/* Get Endpoints values*/
@@ -858,7 +862,6 @@ again:
 
 				usbDevice[reader_index].ccid.sIFD_serial_number = NULL;
 				usbDevice[reader_index].ccid.gemalto_firmware_features = NULL;
-				usbDevice[reader_index].ccid.zlp = FALSE;
 				if (desc.iSerialNumber)
 				{
 					unsigned char serial[128];
@@ -980,7 +983,7 @@ again:
 				}
 
 				// Create thread for card detection
-				r = pthread_create(&usbDevice[reader_index].hThread, NULL, CardDetectionThread, &reader_index);
+				r = pthread_create(&usbDevice[reader_index].hThread, NULL, CardDetectionThread, (void *) (intptr_t) reader_index);
 				if (r != 0)
 				{
 					pthread_mutex_destroy(usbDevice[reader_index].pTransferLock);
@@ -1000,6 +1003,20 @@ again:
 end:
 	if (usbDevice[reader_index].dev_handle == NULL)
 	{
+		/* free the libusb allocated list & devices */
+		libusb_free_device_list(devs, 1);
+
+#ifdef __APPLE__
+		/* give some time to libusb to detect the new USB devices on Mac OS X */
+		if (count_libusb > 0)
+		{
+			count_libusb--;
+			DEBUG_INFO2("Wait after libusb: %d", count_libusb);
+			nanosleep(&sleep_time, NULL);
+
+			goto again_libusb;
+		}
+#endif
 		close_libusb_if_needed();
 		if (claim_failed)
 			return STATUS_COMM_ERROR;
@@ -1041,16 +1058,6 @@ status_t WriteUSB(unsigned int reader_index, unsigned int length,
 	(void)snprintf(debug_header, sizeof(debug_header), "-> %06X ",
 		(int)reader_index);
 
-	if (usbDevice[reader_index].ccid.zlp)
-	{ /* Zero Length Packet */
-		int dummy_length;
-
-		/* try to read a ZLP so transfer length = 0
-		 * timeout of 1 ms */
-		(void)libusb_bulk_transfer(usbDevice[reader_index].dev_handle,
-			usbDevice[reader_index].bulk_in, NULL, 0, &dummy_length, 1);
-	}
-
 	// Fix APG8201 and ACR85 ICC cannot receive command properly
 	// Add delay for APG8201 and ACR85 ICC
 	if ((usbDevice[reader_index].ccid.readerID == ACS_APG8201) ||
@@ -1059,6 +1066,9 @@ status_t WriteUSB(unsigned int reader_index, unsigned int length,
 		delayed = TRUE;
 	}
 
+	// Workaround for ACR122U reader
+	usbDevice[reader_index].last_write_size = length;
+
 	// Send command by dividing number of packets
 	pos = 0;
 	while (length > 0)
@@ -1078,9 +1088,9 @@ status_t WriteUSB(unsigned int reader_index, unsigned int length,
 		{
 			DEBUG_CRITICAL5("write failed (%d/%d): %d %s",
 				usbDevice[reader_index].bus_number,
-				usbDevice[reader_index].device_address, rv, strerror(errno));
+				usbDevice[reader_index].device_address, rv, libusb_error_name(rv));
 
-			if ((ENODEV == errno) || (LIBUSB_ERROR_NO_DEVICE == rv))
+			if (LIBUSB_ERROR_NO_DEVICE == rv)
 				return STATUS_NO_SUCH_DEVICE;
 
 			return STATUS_UNSUCCESSFUL;
@@ -1139,9 +1149,9 @@ status_t ReadUSB(unsigned int reader_index, unsigned int * length,
 				*length = 0;
 				DEBUG_CRITICAL5("read failed (%d/%d): %d %s",
 					usbDevice[reader_index].bus_number,
-					usbDevice[reader_index].device_address, rv, strerror(errno));
+					usbDevice[reader_index].device_address, rv, libusb_error_name(rv));
 
-				if ((ENODEV == errno) || (LIBUSB_ERROR_NO_DEVICE == rv))
+				if (LIBUSB_ERROR_NO_DEVICE == rv)
 					return STATUS_NO_SUCH_DEVICE;
 
 				return STATUS_UNSUCCESSFUL;
@@ -1198,9 +1208,9 @@ read_again:
 		*length = 0;
 		DEBUG_CRITICAL5("read failed (%d/%d): %d %s",
 			usbDevice[reader_index].bus_number,
-			usbDevice[reader_index].device_address, rv, strerror(errno));
+			usbDevice[reader_index].device_address, rv, libusb_error_name(rv));
 
-		if ((ENODEV == errno) || (LIBUSB_ERROR_NO_DEVICE == rv))
+		if (LIBUSB_ERROR_NO_DEVICE == rv)
 			return STATUS_NO_SUCH_DEVICE;
 
 		return STATUS_UNSUCCESSFUL;
@@ -1214,6 +1224,12 @@ read_again:
 	if ((*length >= BSEQ_OFFSET)
 		&& (buffer[BSEQ_OFFSET] < *ccid_descriptor->pbSeq -1))
 	{
+		// Workaround for ACR122U reader
+		if ((ccid_descriptor->readerID == ACS_ACR122U) &&
+			(usbDevice[reader_index].last_write_size > 64) &&
+			(buffer[BSEQ_OFFSET] == *ccid_descriptor->pbSeq -2))
+			return STATUS_SUCCESS;
+
 		duplicate_frame++;
 		if (duplicate_frame > 10)
 		{
@@ -1608,7 +1624,7 @@ int ControlUSB(int reader_index, int requesttype, int request, int value,
 	{
 		DEBUG_CRITICAL5("control failed (%d/%d): %d %s",
 			usbDevice[reader_index].bus_number,
-			usbDevice[reader_index].device_address, ret, strerror(errno));
+			usbDevice[reader_index].device_address, ret, libusb_error_name(ret));
 
 		return ret;
 	}
@@ -1735,7 +1751,7 @@ int InterruptRead(int reader_index, int timeout /* in ms */)
 			/* if libusb_interrupt_transfer() times out we get EILSEQ or EAGAIN */
 			DEBUG_COMM4("InterruptRead (%d/%d): %s",
 				usbDevice[reader_index].bus_number,
-				usbDevice[reader_index].device_address, strerror(errno));
+				usbDevice[reader_index].device_address, libusb_error_name(ret));
 			return_value = IFD_COMMUNICATION_ERROR;
 	}
 
@@ -2021,7 +2037,6 @@ static int Multi_InterruptRead(int reader_index, int timeout /* in ms */)
 	interrupt_mask = 0x02 << (2 * (usbDevice[reader_index].ccid.bCurrentSlotIndex % 4));
 
 	/* Wait until the condition is signaled or a timeout occurs */
-	pthread_mutex_lock(&msExt->mutex);
 	gettimeofday(&local_time, NULL);
 	cond_wait_until.tv_sec = local_time.tv_sec;
 	cond_wait_until.tv_nsec = local_time.tv_usec * 1000;
@@ -2030,6 +2045,8 @@ static int Multi_InterruptRead(int reader_index, int timeout /* in ms */)
 	cond_wait_until.tv_nsec += 1000000 * (timeout % 1000);
 
 again:
+	pthread_mutex_lock(&msExt->mutex);
+
 	rv = pthread_cond_timedwait(&msExt->condition, &msExt->mutex,
 		&cond_wait_until);
 
@@ -2150,7 +2167,7 @@ static struct usbDevice_MultiSlot_Extension *Multi_CreateNextSlot(int physical_r
 // Card detection thread
 static void *CardDetectionThread(void *pParam)
 {
-	int reader_index = *((int *) pParam);
+	int reader_index = (int) pParam;
 	int rv = 0;
 	int status = 0;
 	int actual_length = 0;
@@ -2240,6 +2257,15 @@ static void *CardDetectionThread(void *pParam)
 
 			libusb_free_transfer(transfer);
 
+			/* Break if the device was disconnected. */
+			if (status == LIBUSB_TRANSFER_NO_DEVICE)
+			{
+				DEBUG_COMM3("%d/%d: Device disconnected",
+					usbDevice[reader_index].bus_number,
+					usbDevice[reader_index].device_address);
+				break;
+			}
+
 			switch (status)
 			{
 				case LIBUSB_TRANSFER_COMPLETED:
@@ -2311,6 +2337,7 @@ static void *CardDetectionThread(void *pParam)
 		usbDevice[reader_index].bus_number,
 		usbDevice[reader_index].device_address);
 
-	return ((void *) 0);
+	pthread_exit(NULL);
+	return NULL;
 }
 #endif
