@@ -2,7 +2,7 @@
     commands.c: Commands sent to the card
     Copyright (C) 2003-2010   Ludovic Rousseau
     Copyright (C) 2005 Martin Paljak
-    Copyright (C) 2010-2014   Advanced Card Systems Ltd.
+    Copyright (C) 2010-2015   Advanced Card Systems Ltd.
 
     This library is free software; you can redistribute it and/or
     modify it under the terms of the GNU Lesser General Public
@@ -20,7 +20,7 @@
 */
 
 /*
- * $Id: commands.c 6975 2014-09-04 11:33:05Z rousseau $
+ * $Id$
  */
 
 #include <config.h>
@@ -49,6 +49,7 @@
 #include "defs.h"
 #include "ccid_ifdhandler.h"
 #include "debug.h"
+#include "utils.h"
 
 /* All the pinpad readers I used are more or less bogus
  * I use code to change the user command and make the firmware happy */
@@ -179,7 +180,7 @@ RESPONSECODE CmdPowerOn(unsigned int reader_index, unsigned int * nlength,
 
 			/* Status Information? */
 			if (0x40 == tmp[0])
-				ccid_error(tmp[2], __FILE__, __LINE__, __FUNCTION__);
+				ccid_error(PCSC_LOG_ERROR, tmp[2], __FILE__, __LINE__, __FUNCTION__);
 			return IFD_COMMUNICATION_ERROR;
 		}
 
@@ -246,7 +247,7 @@ again:
 
 	if (buffer[STATUS_OFFSET] & CCID_COMMAND_FAILED)
 	{
-		ccid_error(buffer[ERROR_OFFSET], __FILE__, __LINE__, __FUNCTION__);    /* bError */
+		ccid_error(PCSC_LOG_ERROR, buffer[ERROR_OFFSET], __FILE__, __LINE__, __FUNCTION__);    /* bError */
 
 		if (0xBB == buffer[ERROR_OFFSET] &&	/* Protocol error in EMV mode */
 			((GEMPC433 == ccid_descriptor->readerID)
@@ -325,7 +326,19 @@ RESPONSECODE SecurePINVerify(unsigned int reader_index,
 	if (TxLength < 19+4 /* 4 = APDU size */)	/* command too short? */
 	{
 		DEBUG_INFO3("Command too short: %d < %d", TxLength, 19+4);
-		return IFD_NOT_SUPPORTED;
+
+		/* 6B 80: Invalid parameter in passed structure */
+		if (*RxLength < 2)
+		{
+			return IFD_ERROR_INSUFFICIENT_BUFFER;
+		}
+		else
+		{
+			RxBuffer[0] = 0x6B;
+			RxBuffer[1] = 0x80;
+			*RxLength = 2;
+			return IFD_SUCCESS;
+		}
 	}
 
 	/* On little endian machines we are all set. */
@@ -345,7 +358,19 @@ RESPONSECODE SecurePINVerify(unsigned int reader_index,
 	if (dw2i(TxBuffer, 15) + 19 != TxLength) /* ulDataLength field coherency */
 	{
 		DEBUG_INFO3("Wrong lengths: %d %d", dw2i(TxBuffer, 15) + 19, TxLength);
-		return IFD_NOT_SUPPORTED;
+
+		/* 6B 80: Invalid parameter in passed structure */
+		if (*RxLength < 2)
+		{
+			return IFD_ERROR_INSUFFICIENT_BUFFER;
+		}
+		else
+		{
+			RxBuffer[0] = 0x6B;
+			RxBuffer[1] = 0x80;
+			*RxLength = 2;
+			return IFD_SUCCESS;
+		}
 	}
 
 	/* make sure bEntryValidationCondition is valid
@@ -659,7 +684,19 @@ RESPONSECODE SecurePINModify(unsigned int reader_index,
 	if (TxLength < 24+4 /* 4 = APDU size */) /* command too short? */
 	{
 		DEBUG_INFO3("Command too short: %d < %d", TxLength, 24+4);
-		return IFD_NOT_SUPPORTED;
+
+		/* 6B 80: Invalid parameter in passed structure */
+		if (*RxLength < 2)
+		{
+			return IFD_ERROR_INSUFFICIENT_BUFFER;
+		}
+		else
+		{
+			RxBuffer[0] = 0x6B;
+			RxBuffer[1] = 0x80;
+			*RxLength = 2;
+			return IFD_SUCCESS;
+		}
 	}
 
 	/* On little endian machines we are all set. */
@@ -680,7 +717,19 @@ RESPONSECODE SecurePINModify(unsigned int reader_index,
 	if (dw2i(TxBuffer, 20) + 24 != TxLength) /* ulDataLength field coherency */
 	{
 		DEBUG_INFO3("Wrong lengths: %d %d", dw2i(TxBuffer, 20) + 24, TxLength);
-		return IFD_NOT_SUPPORTED;
+
+		/* 6B 80: Invalid parameter in passed structure */
+		if (*RxLength < 2)
+		{
+			return IFD_ERROR_INSUFFICIENT_BUFFER;
+		}
+		else
+		{
+			RxBuffer[0] = 0x6B;
+			RxBuffer[1] = 0x80;
+			*RxLength = 2;
+			return IFD_SUCCESS;
+		}
 	}
 
 	/* Make sure in the beginning if bNumberMessage is valid or not.
@@ -688,7 +737,19 @@ RESPONSECODE SecurePINModify(unsigned int reader_index,
 	if ((TxBuffer[11] > 3) && (TxBuffer[11] != 0xFF))
 	{
 		DEBUG_INFO2("Wrong bNumberMessage: %d", TxBuffer[11]);
-		return IFD_NOT_SUPPORTED;
+
+		/* 6B 80: Invalid parameter in passed structure */
+		if (*RxLength < 2)
+		{
+			return IFD_ERROR_INSUFFICIENT_BUFFER;
+		}
+		else
+		{
+			RxBuffer[0] = 0x6B;
+			RxBuffer[1] = 0x80;
+			*RxLength = 2;
+			return IFD_SUCCESS;
+		}
 	}
 
 	/* Make sure bEntryValidationCondition is valid
@@ -884,6 +945,21 @@ end:
 RESPONSECODE CmdEscape(unsigned int reader_index,
 	const unsigned char TxBuffer[], unsigned int TxLength,
 	unsigned char RxBuffer[], unsigned int *RxLength, unsigned int timeout)
+{
+	return CmdEscapeCheck(reader_index, TxBuffer, TxLength, RxBuffer, RxLength,
+		timeout, FALSE);
+} /* CmdEscape */
+
+
+/*****************************************************************************
+ *
+ *					Escape (with check of gravity)
+ *
+ ****************************************************************************/
+RESPONSECODE CmdEscapeCheck(unsigned int reader_index,
+	const unsigned char TxBuffer[], unsigned int TxLength,
+	unsigned char RxBuffer[], unsigned int *RxLength, unsigned int timeout,
+	int mayfail)
 {
 	unsigned char *cmd_in, *cmd_out;
 	status_t res;
@@ -980,7 +1056,9 @@ time_request:
 
 	if (cmd_out[STATUS_OFFSET] & CCID_COMMAND_FAILED)
 	{
-		ccid_error(cmd_out[ERROR_OFFSET], __FILE__, __LINE__, __FUNCTION__);    /* bError */
+		/* mayfail: the error may be expected and not fatal */
+		ccid_error(mayfail ? PCSC_LOG_INFO : PCSC_LOG_ERROR,
+			cmd_out[ERROR_OFFSET], __FILE__, __LINE__, __FUNCTION__);    /* bError */
 		return_value = IFD_COMMUNICATION_ERROR;
 	}
 
@@ -998,7 +1076,7 @@ end:
 		ccid_descriptor -> readTimeout = old_read_timeout;
 
 	return return_value;
-} /* Escape */
+} /* EscapeCheck */
 
 
 /*****************************************************************************
@@ -1082,7 +1160,7 @@ RESPONSECODE CmdPowerOff(unsigned int reader_index)
 
 	if (cmd[STATUS_OFFSET] & CCID_COMMAND_FAILED)
 	{
-		ccid_error(cmd[ERROR_OFFSET], __FILE__, __LINE__, __FUNCTION__);    /* bError */
+		ccid_error(PCSC_LOG_ERROR, cmd[ERROR_OFFSET], __FILE__, __LINE__, __FUNCTION__);    /* bError */
 		return_value = IFD_COMMUNICATION_ERROR;
 	}
 
@@ -1203,7 +1281,7 @@ again_status:
 		&& (buffer[ERROR_OFFSET] != 0xFE))
 	{
 		return_value = IFD_COMMUNICATION_ERROR;
-		ccid_error(buffer[ERROR_OFFSET], __FILE__, __LINE__, __FUNCTION__);    /* bError */
+		ccid_error(PCSC_LOG_ERROR, buffer[ERROR_OFFSET], __FILE__, __LINE__, __FUNCTION__);    /* bError */
 	}
 
 	return return_value;
@@ -1449,7 +1527,7 @@ time_request_ICCD_B:
 
 			case 0x40:
 				/* Status Information */
-				ccid_error(rx_buffer[2], __FILE__, __LINE__, __FUNCTION__);
+				ccid_error(PCSC_LOG_ERROR, rx_buffer[2], __FILE__, __LINE__, __FUNCTION__);
 				return IFD_COMMUNICATION_ERROR;
 
 			case 0x80:
@@ -1508,22 +1586,30 @@ time_request:
 
 	if (cmd[STATUS_OFFSET] & CCID_COMMAND_FAILED)
 	{
-		ccid_error(cmd[ERROR_OFFSET], __FILE__, __LINE__, __FUNCTION__);    /* bError */
+		ccid_error(PCSC_LOG_ERROR, cmd[ERROR_OFFSET], __FILE__, __LINE__, __FUNCTION__);    /* bError */
 		switch (cmd[ERROR_OFFSET])
 		{
 			case 0xFF:	// Abort
 				// ACR83U, ACR85 and APG8201
 				// Invalid parameter in PIN verification/modification data structure
 				if (*rx_length < 2)
-					return IFD_COMMUNICATION_ERROR;
+					return IFD_ERROR_INSUFFICIENT_BUFFER;
 				rx_buffer[0]= 0x6B;
 				rx_buffer[1]= 0x80;
 				*rx_length = 2;
 				return IFD_SUCCESS;
 
+			case 0x84:	// Two "new PIN" entries do not match.
+				if (*rx_length < 2)
+					return IFD_ERROR_INSUFFICIENT_BUFFER;
+				rx_buffer[0]= 0x64;
+				rx_buffer[1]= 0x02;
+				*rx_length = 2;
+				return IFD_SUCCESS;
+
 			case 0xEF:	/* cancel */
 				if (*rx_length < 2)
-					return IFD_COMMUNICATION_ERROR;
+					return IFD_ERROR_INSUFFICIENT_BUFFER;
 				rx_buffer[0]= 0x64;
 				rx_buffer[1]= 0x01;
 				*rx_length = 2;
@@ -1531,7 +1617,7 @@ time_request:
 
 			case 0xF0:	/* timeout */
 				if (*rx_length < 2)
-					return IFD_COMMUNICATION_ERROR;
+					return IFD_ERROR_INSUFFICIENT_BUFFER;
 				rx_buffer[0]= 0x64;
 				rx_buffer[1]= 0x00;
 				*rx_length = 2;
@@ -2274,7 +2360,7 @@ RESPONSECODE SetParameters(unsigned int reader_index, char protocol,
 
 	if (cmd[STATUS_OFFSET] & CCID_COMMAND_FAILED)
 	{
-		ccid_error(cmd[ERROR_OFFSET], __FILE__, __LINE__, __FUNCTION__);    /* bError */
+		ccid_error(PCSC_LOG_ERROR, cmd[ERROR_OFFSET], __FILE__, __LINE__, __FUNCTION__);    /* bError */
 		if (0x00 == cmd[ERROR_OFFSET])	/* command not supported */
 			return IFD_NOT_SUPPORTED;
 		else
