@@ -18,7 +18,7 @@
 */
 
 /*
- * $Id: commands.h 6783 2013-10-24 09:36:52Z rousseau $
+ * $Id$
  */
 
 #define SIZE_GET_SLOT_STATUS 10
@@ -41,6 +41,11 @@ RESPONSECODE CmdEscape(unsigned int reader_index,
 	const unsigned char TxBuffer[], unsigned int TxLength,
 	unsigned char RxBuffer[], unsigned int *RxLength, unsigned int timeout);
 
+RESPONSECODE CmdEscapeCheck(unsigned int reader_index,
+	const unsigned char TxBuffer[], unsigned int TxLength,
+	unsigned char RxBuffer[], unsigned int *RxLength, unsigned int timeout,
+	int mayfail);
+
 RESPONSECODE CmdPowerOff(unsigned int reader_index);
 
 RESPONSECODE CmdGetSlotStatus(unsigned int reader_index,
