@@ -1,7 +1,7 @@
 /*
     ifdhandler.c: IFDH API
     Copyright (C) 2003-2010   Ludovic Rousseau
-    Copyright (C) 2009-2014   Advanced Card Systems Ltd.
+    Copyright (C) 2009-2015   Advanced Card Systems Ltd.
 
     This library is free software; you can redistribute it and/or
     modify it under the terms of the GNU Lesser General Public
@@ -18,7 +18,7 @@
 	Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */
 
-/* $Id: ifdhandler.c 6977 2014-09-04 11:36:54Z rousseau $ */
+/* $Id$ */
 
 #include <config.h>
 
@@ -86,6 +86,9 @@ static unsigned int T0_card_timeout(double f, double d, int TC1, int TC2,
 static unsigned int T1_card_timeout(double f, double d, int TC1, int BWI,
 	int CWI, int clock_frequency);
 static int get_IFSC(ATR_t *atr, int *i);
+static RESPONSECODE process_spe_ppdu(unsigned int reader_index,
+	unsigned char TxBuffer[], unsigned int TxLength,
+	unsigned char RxBuffer[], unsigned int *RxLength);
 
 
 static RESPONSECODE CreateChannelByNameOrChannel(DWORD Lun,
@@ -383,7 +386,7 @@ static RESPONSECODE IFDHSleep(DWORD Lun, int timeout)
 	 * TAG_IFD_POLLING_THREAD_KILLABLE then we could use a much longer delay
 	 * and be killed before pcscd exits
 	 */
-	(void)usleep(timeout);
+	(void)usleep(timeout * 1000);
 	return IFD_SUCCESS;
 }
 
@@ -413,7 +416,7 @@ EXTERNAL RESPONSECODE IFDHGetCapabilities(DWORD Lun, DWORD Tag,
 	 * ignore Lun.
 	 *
 	 * Tag - the tag for the information requested example: TAG_IFD_ATR -
-	 * return the Atr and it's size (required). these tags are defined in
+	 * return the Atr and its size (required). these tags are defined in
 	 * ifdhandler.h
 	 *
 	 * Length - the length of the returned data Value - the value of the
@@ -525,6 +528,9 @@ EXTERNAL RESPONSECODE IFDHGetCapabilities(DWORD Lun, DWORD Tag,
 						(ACS_ACR1251K_DUAL_READER == readerID) ||
 						(ACS_ACR1252_1S_CL_READER == readerID))
 						*Value = 2;
+
+					if (FEITIANR502DUAL == readerID)
+						*Value = 3;
 				}
 #endif
 				DEBUG_INFO2("Reader supports %d slot(s)", *Value);
@@ -1258,7 +1264,7 @@ EXTERNAL RESPONSECODE IFDHPowerICC(DWORD Lun, DWORD Action,
 	 * Action - Action to be taken on the card.
 	 *
 	 * IFD_POWER_UP - Power and reset the card if not done so (store the
-	 * ATR and return it and it's length).
+	 * ATR and return it and its length).
 	 *
 	 * IFD_POWER_DOWN - Power down the card if not done already
 	 * (Atr/AtrLength should be zero'd)
@@ -1268,7 +1274,7 @@ EXTERNAL RESPONSECODE IFDHPowerICC(DWORD Lun, DWORD Action,
 	 *
 	 * Atr - Answer to Reset of the card.  The driver is responsible for
 	 * caching this value in case IFDHGetCapabilities is called requesting
-	 * the ATR and it's length.  This should not exceed MAX_ATR_SIZE.
+	 * the ATR and its length.  This should not exceed MAX_ATR_SIZE.
 	 *
 	 * AtrLength - Length of the Atr.  This should not exceed
 	 * MAX_ATR_SIZE.
@@ -1672,6 +1678,16 @@ EXTERNAL RESPONSECODE IFDHTransmitToICC(DWORD Lun, SCARD_IO_HEADER SendPci,
 		}
 	}
 
+	/* Process SPE pseudo APDU (PC/SC v2.02.02 Part 10 Supplement). */
+	if ((TxLength > 3)
+		&& (memcmp(TxBuffer, "\xFF\xC2\x01", 3) == 0))
+	{
+		rx_length = *RxLength;
+		return_value = process_spe_ppdu(reader_index, TxBuffer, TxLength,
+			RxBuffer, &rx_length);
+		goto err;
+	}
+
 	rx_length = *RxLength;
 	return_value = CcidSlots[reader_index].pXfrBlock(reader_index, TxLength, TxBuffer, &rx_length,
 		RxBuffer, SendPci.Protocol);
@@ -1885,7 +1901,8 @@ EXTERNAL RESPONSECODE IFDHControl(DWORD Lun, DWORD dwControlCode,
 		iBytesReturned += sizeof(PCSC_TLV_STRUCTURE);
 
 		// APG8201Z supports vendor specific feature
-		if (ACS_APG8201Z == ccid_descriptor -> readerID)
+		if ((ACS_APG8201Z == ccid_descriptor -> readerID)
+			|| (ACS_APG8201Z2 == ccid_descriptor -> readerID))
 		{
 			pcsc_tlv -> tag = 0x80;
 			pcsc_tlv -> length = 0x04; /* always 0x04 */
@@ -2056,7 +2073,8 @@ EXTERNAL RESPONSECODE IFDHControl(DWORD Lun, DWORD dwControlCode,
 		/* ACR83, APG8201 and APG8201Z */
 		if ((ACS_ACR83U == ccid_descriptor -> readerID)
 			|| (ACS_APG8201 == ccid_descriptor -> readerID)
-			|| (ACS_APG8201Z == ccid_descriptor -> readerID))
+			|| (ACS_APG8201Z == ccid_descriptor -> readerID)
+			|| (ACS_APG8201Z2 == ccid_descriptor -> readerID))
 		{
 			/* bMinPINSize */
 			RxBuffer[p++] = PCSCv2_PART10_PROPERTY_bMinPINSize;
@@ -2077,9 +2095,10 @@ EXTERNAL RESPONSECODE IFDHControl(DWORD Lun, DWORD dwControlCode,
 		/* bPPDUSupport */
 		RxBuffer[p++] = PCSCv2_PART10_PROPERTY_bPPDUSupport;
 		RxBuffer[p++] = 1;	/* length */
-		RxBuffer[p++] = 1;
+		RxBuffer[p++] = 0x03;
 			/* bit0: PPDU is supported over SCardControl using
-			 * FEATURE_CCID_ESC_COMMAND */
+			 * FEATURE_CCID_ESC_COMMAND
+			 * bit1: PPDU is supported over SCardTransmit */
 
 		/* wIdVendor */
 		{
@@ -2189,7 +2208,8 @@ EXTERNAL RESPONSECODE IFDHControl(DWORD Lun, DWORD dwControlCode,
 	if ((ACS_ACR83U == ccid_descriptor -> readerID) ||
 		(ACS_ACR85_PINPAD_READER_ICC == ccid_descriptor -> readerID) ||
 		(ACS_APG8201 == ccid_descriptor -> readerID) ||
-		(ACS_APG8201Z == ccid_descriptor -> readerID))
+		(ACS_APG8201Z == ccid_descriptor -> readerID) ||
+		(ACS_APG8201Z2 == ccid_descriptor -> readerID))
 	{
 		// Get firmware version
 		if (IOCTL_SMARTCARD_GET_FIRMWARE_VERSION == dwControlCode)
@@ -2909,3 +2929,333 @@ static int get_IFSC(ATR_t *atr, int *idx)
 	return ifsc;
 } /* get_IFSC */
 
+
+static RESPONSECODE process_spe_ppdu(unsigned int reader_index,
+	unsigned char TxBuffer[], unsigned int TxLength,
+	unsigned char RxBuffer[], unsigned int *RxLength)
+{
+	RESPONSECODE ret = IFD_SUCCESS;
+	_ccid_descriptor *ccid_descriptor = get_ccid_descriptor(reader_index);
+	int supported = FALSE;
+	unsigned char buffer[43];
+	unsigned int length = 0;
+	unsigned int tmp = 0;
+
+	/* P2: Feature Number */
+	switch (TxBuffer[3])
+	{
+	case 0:	/* GET_FEATURE_REQUEST */
+		supported = TRUE;
+
+		if (ccid_descriptor->bPINSupport & CCID_CLASS_PIN_VERIFY)
+		{
+			buffer[length++] = FEATURE_VERIFY_PIN_DIRECT;
+		}
+
+		if (ccid_descriptor->bPINSupport & CCID_CLASS_PIN_MODIFY)
+		{
+			buffer[length++] = FEATURE_MODIFY_PIN_DIRECT;
+		}
+
+		if (ccid_descriptor->bPINSupport)
+		{
+			buffer[length++] = FEATURE_IFD_PIN_PROPERTIES;
+		}
+
+		buffer[length++] = FEATURE_GET_TLV_PROPERTIES;
+		buffer[length++] = FEATURE_CCID_ESC_COMMAND;
+
+		if ((ccid_descriptor->readerID == ACS_APG8201Z)
+			|| (ccid_descriptor->readerID == ACS_APG8201Z2))
+		{
+			buffer[length++] = 0x80;
+		}
+
+		/* 90 00: Feature executed successfully. */
+		if (*RxLength < length + 2)
+		{
+			ret = IFD_ERROR_INSUFFICIENT_BUFFER;
+		}
+		else
+		{
+			memcpy(RxBuffer, buffer, length);
+			length += 2;
+			RxBuffer[length - 2] = 0x90;
+			RxBuffer[length - 1] = 0x00;
+			*RxLength = length;
+		}
+		break;
+
+	case FEATURE_VERIFY_PIN_DIRECT:
+		if (ccid_descriptor->bPINSupport & CCID_CLASS_PIN_VERIFY)
+		{
+			supported = TRUE;
+
+			/* Check the length and Lc. */
+			/* Minimum Length: CLA + INS + P1 + P2 + Lc + data */
+			if ((TxLength < 6) || (TxBuffer[4] != TxLength - 5))
+			{
+				/* 67 00: Wrong length; no further indication */
+				if (*RxLength < 2)
+				{
+					ret = IFD_ERROR_INSUFFICIENT_BUFFER;
+				}
+				else
+				{
+					RxBuffer[0] = 0x67;
+					RxBuffer[1] = 0x00;
+					*RxLength = 2;
+				}
+			}
+			else
+			{
+				tmp = *RxLength;
+				ret = SecurePINVerify(reader_index, TxBuffer + 5, TxLength - 5,
+					RxBuffer, &tmp);
+				*RxLength = tmp;
+			}
+		}
+		break;
+
+	case FEATURE_MODIFY_PIN_DIRECT:
+		if (ccid_descriptor->bPINSupport & CCID_CLASS_PIN_MODIFY)
+		{
+			supported = TRUE;
+
+			/* Check the length and Lc. */
+			/* Minimum Length: CLA + INS + P1 + P2 + Lc + data */
+			if ((TxLength < 6) || (TxBuffer[4] != TxLength - 5))
+			{
+				/* 67 00: Wrong length; no further indication */
+				if (*RxLength < 2)
+				{
+					ret = IFD_ERROR_INSUFFICIENT_BUFFER;
+				}
+				else
+				{
+					RxBuffer[0] = 0x67;
+					RxBuffer[1] = 0x00;
+					*RxLength = 2;
+				}
+			}
+			else
+			{
+				tmp = *RxLength;
+				ret = SecurePINModify(reader_index, TxBuffer + 5, TxLength - 5,
+					RxBuffer, &tmp);
+				*RxLength = tmp;
+			}
+		}
+		break;
+
+	case FEATURE_IFD_PIN_PROPERTIES:
+		if (ccid_descriptor->bPINSupport)
+		{
+			supported = TRUE;
+
+			if (*RxLength < 6)
+			{
+				ret = IFD_ERROR_INSUFFICIENT_BUFFER;
+			}
+			else
+			{
+				RxBuffer[0] = ccid_descriptor->wLcdLayout & 0xFF;
+				RxBuffer[1] = (ccid_descriptor->wLcdLayout >> 8) & 0xFF;
+				RxBuffer[2] = 0x07;	/* bEntryValidationCondition */
+				RxBuffer[3] = 0x00;	/* bTimeOut2 */
+				RxBuffer[4] = 0x90;
+				RxBuffer[5] = 0x00;
+				*RxLength = 6;
+			}
+		}
+		break;
+
+	case FEATURE_GET_TLV_PROPERTIES:
+		supported = TRUE;
+
+		/* wLcdLayout */
+		buffer[length++] = PCSCv2_PART10_PROPERTY_wLcdLayout;
+		buffer[length++] = 2;
+		buffer[length++] = ccid_descriptor->wLcdLayout & 0xFF;
+		buffer[length++] = (ccid_descriptor->wLcdLayout >> 8) & 0xFF;
+
+		if (ccid_descriptor->wLcdLayout)
+		{
+			/* wLcdMaxCharacters */
+			buffer[length++] = PCSCv2_PART10_PROPERTY_wLcdMaxCharacters;
+			buffer[length++] = 2;
+			tmp = ccid_descriptor->wLcdLayout & 0xFF;
+			buffer[length++] = tmp & 0xFF;
+			buffer[length++] = (tmp >> 8) & 0xFF;
+
+			/* wLcdMaxLines */
+			buffer[length++] = PCSCv2_PART10_PROPERTY_wLcdMaxLines;
+			buffer[length++] = 2;
+			tmp = (ccid_descriptor->wLcdLayout >> 8) & 0xFF;
+			buffer[length++] = tmp & 0xFF;
+			buffer[length++] = (tmp >> 8) & 0xFF;
+		}
+
+		/* bTimeOut2 */
+		buffer[length++] = PCSCv2_PART10_PROPERTY_bTimeOut2;
+		buffer[length++] = 1;
+		buffer[length++] = 0x00;
+
+		/* ACR83, APG8201 and APG8201Z. */
+		if ((ccid_descriptor->readerID == ACS_ACR83U)
+			|| (ccid_descriptor->readerID == ACS_APG8201)
+			|| (ccid_descriptor->readerID == ACS_APG8201Z)
+			|| (ccid_descriptor->readerID == ACS_APG8201Z2))
+		{
+			/* bMinPINSize */
+			buffer[length++] = PCSCv2_PART10_PROPERTY_bMinPINSize;
+			buffer[length++] = 1;
+			buffer[length++] = 1;
+
+			/* bMaxPINSize */
+			buffer[length++] = PCSCv2_PART10_PROPERTY_bMaxPINSize;
+			buffer[length++] = 1;
+			buffer[length++] = 16;
+
+			/* bEntryValidationCondition */
+			buffer[length++] = PCSCv2_PART10_PROPERTY_bEntryValidationCondition;
+			buffer[length++] = 1;
+			buffer[length++] = 0x07;
+		}
+
+		/* bPPDUSupport */
+		buffer[length++] = PCSCv2_PART10_PROPERTY_bPPDUSupport;
+		buffer[length++] = 1;
+		buffer[length++] = 0x03;
+
+		/* wIdVendor */
+		tmp = (ccid_descriptor->readerID >> 16) & 0xFFFF;
+		buffer[length++] = PCSCv2_PART10_PROPERTY_wIdVendor;
+		buffer[length++] = 2;
+		buffer[length++] = tmp & 0xFF;
+		buffer[length++] = (tmp >> 8) & 0xFF;
+
+		/* wIdProduct */
+		tmp = ccid_descriptor->readerID & 0xFFFF;
+		buffer[length++] = PCSCv2_PART10_PROPERTY_wIdProduct;
+		buffer[length++] = 2;
+		buffer[length++] = tmp & 0xFF;
+		buffer[length++] = (tmp >> 8) & 0xFF;
+
+		/* dwMaxAPDUDataSize */
+		/* Short APDU only by default */
+		tmp = 0;
+
+		/* Reader is TPDU or extended APDU. */
+		if ((ccid_descriptor->dwFeatures & CCID_CLASS_EXTENDED_APDU)
+			|| (ccid_descriptor->dwFeatures & CCID_CLASS_TPDU))
+		{
+			tmp = 0x10000;
+		}
+
+		buffer[length++] = PCSCv2_PART10_PROPERTY_dwMaxAPDUDataSize;
+		buffer[length++] = 4;
+		buffer[length++] = tmp & 0xFF;
+		buffer[length++] = (tmp >> 8) & 0xFF;
+		buffer[length++] = (tmp >> 16) & 0xFF;
+		buffer[length++] = (tmp >> 24) & 0xFF;
+
+		/* 90 00: Feature executed successfully. */
+		if (*RxLength < length + 2)
+		{
+			ret = IFD_ERROR_INSUFFICIENT_BUFFER;
+		}
+		else
+		{
+			memcpy(RxBuffer, buffer, length);
+			length += 2;
+			RxBuffer[length - 2] = 0x90;
+			RxBuffer[length - 1] = 0x00;
+			*RxLength = length;
+		}
+		break;
+
+	case FEATURE_CCID_ESC_COMMAND:
+		supported = TRUE;
+
+		/* Check the length and Lc. */
+		/* Minimum Length: CLA + INS + P1 + P2 + Lc + data */
+		if ((TxLength < 6) || (TxBuffer[4] != TxLength - 5))
+		{
+			/* 67 00: Wrong length; no further indication */
+			if (*RxLength < 2)
+			{
+				ret = IFD_ERROR_INSUFFICIENT_BUFFER;
+			}
+			else
+			{
+				RxBuffer[0] = 0x67;
+				RxBuffer[1] = 0x00;
+				*RxLength = 2;
+			}
+		}
+		else
+		{
+			tmp = *RxLength;
+			ret = CmdEscape(reader_index, TxBuffer + 5, TxLength - 5, RxBuffer,
+				&tmp, -1);
+			if (ret == IFD_SUCCESS)
+			{
+				/* 90 00: Feature executed successfully. */
+				if (*RxLength < tmp + 2)
+				{
+					ret = IFD_ERROR_INSUFFICIENT_BUFFER;
+				}
+				else
+				{
+					tmp += 2;
+					RxBuffer[tmp - 2] = 0x90;
+					RxBuffer[tmp - 1] = 0x00;
+					*RxLength = tmp;
+				}
+			}
+		}
+		break;
+
+	case 0x80:
+		if ((ccid_descriptor->readerID == ACS_APG8201Z)
+			|| (ccid_descriptor->readerID == ACS_APG8201Z2))
+		{
+			supported = TRUE;
+
+			/* 90 00: Feature executed successfully. */
+			if (*RxLength < 2)
+			{
+				ret = IFD_ERROR_INSUFFICIENT_BUFFER;
+			}
+			else
+			{
+				RxBuffer[0] = 0x90;
+				RxBuffer[1] = 0x00;
+				*RxLength = 2;
+			}
+		}
+		break;
+
+	default:
+		break;
+	}
+
+	if (!supported)
+	{
+		/* 6A 86: Incorrect value for P2 (requested feature not present) */
+		if (*RxLength < 2)
+		{
+			ret = IFD_ERROR_INSUFFICIENT_BUFFER;
+		}
+		else
+		{
+			RxBuffer[0] = 0x6A;
+			RxBuffer[1] = 0x86;
+			*RxLength = 2;
+		}
+	}
+
+	return ret;
+} /* process_spe_ppdu */
+
